// *****************************************************************************
// * IBM MQSeries Workflow XML Sample Scenario                                 *
// *****************************************************************************
//
//  Licensed Materials - Property of IBM
//  5697-FM3 5655-A96
//  (C) Copyright IBM Corp. 1999-2000
//  US Government Users Restricted Rights - Use, duplication or
//  disclosure restricted by GSA ADP Schedule Contract
//  with IBM Corp.
//
// *****************************************************************************
// *****************************************************************************
//  DISCLAIMER
//  ----------
//  This material contains programming source code for your consideration.
//  These examples have not been thoroughly tested under all conditions.
//  IBM, therefore, cannot guarantee or imply reliability, serviceability,
//  or function of these programs.
//  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
//  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
//  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
//  FOR A PARTICULAR PURPOSE.
// *****************************************************************************
package com.ibm.workflow.upes;

import java.util.Hashtable;
import java.util.Vector;
import org.w3c.dom.NodeList;
import org.w3c.dom.Node;
import org.w3c.dom.Element;
import org.w3c.dom.Text;

/**
 * The <b>Container</b> class is a simple representation of MQSeries
 * Workflow container. It has the ability to navigate through
 * a Element based 'XML parse tree' and to get the
 * container information out of an XML container representation.
 * On the other side it provides an interface to access container
 * members in a natural way and to generate an XML representation
 * of itself.
 * @version 1.0
 */
public class Container 
{
  /************************************************************
   * <b>_memberMap</b> is a Hashtable which contains key-value pairs
   * for every container element.
   * The content is redundant to _rootElement.
   * @see #GetMemberMap
   * @see java.util.Hashtable
   *
  ************************************************************/
  private Hashtable        _memberMap;

  /************************************************************
   * Getter <b>GetMemberMap</b> returns the Hashtable which 
   * contains key/value pairs for every container element!   
   * <b>This hashtable must not be used to change container values!</b>
   * <b>To change container values you have to use GetRootElement</b>
   * @see java.util.Hashtable
   * @see #GetRootElement
   *
  ************************************************************/
  public  Hashtable GetMemberMap()
  {
    return _memberMap;
  }
  /************************************************************
   * <b>_rootElement</b> 
   * is the root of container elements which build
   * up the container. The content is redundant to _memberMap.
   * @see #GetRootElement
   * @see ContainerElement
   *
  ************************************************************/
  private ContainerElement _rootElement;
  /************************************************************
   * Getter <b>GetRootElement</b> returns the root of container 
   * elements which build up the container.
   * @see ContainerElement
   *
  ************************************************************/
  public  ContainerElement GetRootElement()
  {
    return _rootElement;
  }
/************************************************************
 * Constructor, which processes a Element and sets the internal
 * datastructure accordingly.
 *
 * @param containerRoot the root element of the container XML representation.
 * 
 ***********************************************************/
  public Container(Element containerRoot)
  {
    _memberMap = new Hashtable();
    _rootElement = Traverse("",containerRoot);
  }  
/************************************************************
 * The method <b>AsXMLString</b> returns an XML representation of the container.
 * @return  An XML representation of the container.
 * 
 ***********************************************************/
  public String AsXMLString()
  {
    return _rootElement.AsXMLString(" ",false);
  }  
/************************************************************
 * The method <b>Traverse</b> traverses a given Node recursively. 
 * During the traversal it fills the hashtable and the internal 
 * strukture with the values seen during the traversal.
 *
 * @param prefix string which describes the nesting level in a dot-notation.
 * This is needed for accessing nested information via the hashtable.
 * 
 * @param node is the (DOM) Node which is to processed by this method.
 * @return  ContainerElement representing the parsed node.
 * 
 ***********************************************************/
  private ContainerElement Traverse(String prefix, Node node)
  {
    ContainerElement result = null;
    try
    {
      String nestedPrefix = "";
      //Prefix components are seperated by a dot
      if (!prefix.equals(""))
      {
        nestedPrefix = prefix+".";
      }
      //only (DOM) Element Nodes are to be processed
      if (node.getNodeType() == Node.ELEMENT_NODE)
      {
        //key name for a new Hashtable entry consists of prefix.tagName
        String keyName = nestedPrefix+((Element)node).getTagName();
        //for each (DOM) element node a container element is created which represents
        //the subtree beneath the node (including the node itself)
        result = new ContainerElement(((Element)node).getTagName());

        //allSubElements contains subtree
        NodeList allSubElements = node.getChildNodes();

        Tracer.trace("Processing: "+prefix+"-->"+((Element)node).getTagName());
        if (allSubElements.getLength() == 1)
        {
          //only one subnode exists --> it may be a single text node or something else
          if (allSubElements.item(0).getNodeType() == Node.TEXT_NODE)
          {
            String elementData = ((Text)allSubElements.item(0)).getData();
            // TextNodes contains the value 
            // it must be set in the hashtable with the appropiate key
            // and the container elements value representing the given node
            // must be set.
            _memberMap.put(keyName, elementData);
            result.SetValue( elementData);
          }
          else
          {
            // It is not a text node, so it is processed recursively
            result.AddChild(Traverse(keyName,allSubElements.item(0)));
          }
        }
        else
        {
          // there are more than one subnodes
          for (int i = 0; i < allSubElements.getLength(); i++)
          {
            //only (DOM) element nodes are process
            if (allSubElements.item(i).getNodeType() == Node.ELEMENT_NODE)
            {
              result.AddChild(Traverse(keyName,allSubElements.item(i)));
            }
          }
        }
      }
    }
    catch (Exception e)
    {
      Tracer.trace("error: "+e.getMessage());
      e.printStackTrace();
    }
    return result;
  }  

  public void RemovePreDefinedDataMembers()
  {
    Vector childs = _rootElement.GetChildren();
    //- skip pre-defined data members
    int numChilds = childs.size(); 
    for ( int i=0; i < childs.size(); )
    {
      ContainerElement tempEl = (ContainerElement) childs.get( i);
      Tracer.trace(" OutContainerElement:" + tempEl.GetName());
      if ( !tempEl.GetName().startsWith( "_"))
      {
        //- got it
        i++;
      }
      else
      {
        _rootElement.RemoveChild( tempEl);
      }
    }
  }
}      
