/*BEGIN CMVC */
//*************************************************************************
//
//  Workfile: %W%
//  Last update: %E% %U%
//  SCCS path, id: %F% %I%
//
//************************************************************************/
/*END CMVC */
/*BEGIN COPYRIGHT */
//*************************************************************************
//
//  Licensed Materials - Property of IBM
//  WA07 Support Pac - Web Services Process Management Toolkit
//  (c) Copyright IBM Corp. 2001
//  US Government Users Restricted Rights - Use, duplication or
//  disclosure restricted by GSA ADP Schedule Contract
//  with IBM Corp.
//
//************************************************************************/
/*END COPYRIGHT */
/*BEGIN DISCLAIMER */
//*************************************************************************
//  DISCLAIMER
//  ----------
//  This material contains programming source code for your consideration.
//  These examples have not been thoroughly tested under all conditions.
//  IBM, therefore, cannot guarantee or imply reliability, serviceability,
//  or function of these programs.
//  ALL PROGRAMS CONTAINED HEREIN ARE PROVIDED TO YOU "AS IS", WITHOUT ANY
//  WARRANTIES (EXPRESS OR IMPLIED) OR SUPPORT WHATSOEVER, INCLUDING BUT
//  NOT LIMITED TO ANY IMPLIED WARRANTIES OF MERCHANTABILITY OR FITNESS
//  FOR A PARTICULAR PURPOSE.
//************************************************************************/
/*END DISCLAIMER */

package com.ibm.workflow.util.fdl.make;

import com.ibm.workflow.util.fdl.*;
import java.util.*;
import java.io.*;

/**
 *
 * Common base class for all generators operating on FDL parse trees. It deals with FDL parsing
 * (using a <code>com.ibm.workflow.util.fdl.Parser</code>), configuration parsing (using a
 * <code>com.ibm.workflow.util.cfg.Parser</code>), template handling and optional postprocessing (if
 * a filter class is configured). Actual code generation is delegated to an instance implementing the
 * <code>com.ibm.workflow.util.fdl.ParserVisitor</code>.
 *
 * @author  Robert Junghuber (<a href='mailto:junghub@de.ibm.com'>junghub@de.ibm.com</a>)
 * @version 1.2
 *
 */

public abstract class Make
{
  /**
   *
   * Main entry point. <code>args[0]</code> specifies the name of the FDL file, <code>args[1]</code> is the (optional) name
   * of the output file
   *
   */

  protected Make() {}

  protected Make(String[] args) throws Exception
  {
    if (args.length > 0)
    {
      // Remember input and output file name

      src = args[0];
      tgt = args.length < 2 ? null : args[1];

      // Create an FDL parser for the given file

      File input = new File(src);

      com.ibm.workflow.util.fdl.Parser parser = new com.ibm.workflow.util.fdl.Parser(new FileInputStream(input));

      // Construct the FDL parse tree

      cu = parser.CompilationUnit();

      // Lookup (or compose) the file name for the configuration file

      String cfg = args.length < 3 ? null : args[2];

      if (cfg == null) cfg = System.getProperty("FDL2_CFG", "fdl2" + ext() + ".cfg");

      System.out.println(eol + name() + ": Using configuration file \"" + cfg + "\" ...");

      // Create a configuration parser (once)

      if (cfgParser == null) cfgParser = new com.ibm.workflow.util.cfg.Parser(cfg);

      // Fill in substitution values, creating substitution table if none exists.

      Hashtable subst = cfgParser.getParameters("SUBST");

      if (subst == null)
      {
        subst = new Hashtable();

        cfgParser.getTopics().put("SUBST", subst);
      }

      subst.put("InputFileName",  input.getAbsolutePath());
      subst.put("GeneratorName",  name());
      subst.put("GeneratedDate",  (new Date()).toString());
    }
    else throw new Exception(eol +  name() + ": Usage is:" + eol + eol + "java " + getClass().getName() + " " + usage());
  }

  protected final void make(String arg) throws Exception
  {
    boolean success = false;
    File    output  = null;

    try
    {
      // If no output file name is given, compose the file name for the output stream, prevent overwriting input

      if (arg != null) tgt = arg;
      if (tgt == null) tgt = src.substring(0, src.lastIndexOf('.')) + "." + ext();

      if (tgt.equals(src)) throw new IOException("Duplicate file name \"" + tgt + "\"");

      output = new File(tgt);

      String pathName = output.getAbsolutePath();

      pathName = pathName.substring(0, pathName.lastIndexOf(File.separatorChar));

      File path = new File(pathName);

      if (!path.exists()) path.mkdirs();

      out = getPrintWriter(output);

      Hashtable subst = cfgParser.getParameters("SUBST");

      subst.put("OutputFileName", output.getAbsolutePath());

      // Create a parser visitor of the desired type

      ParserVisitor pv = (ParserVisitor)visitor().getConstructor(new Class [] { Make.class })
                                                 .newInstance   (new Object[] { this });
      cu.jjtAccept(pv, null);

      // Open a (buffered) reader on the configured template file. If none is configured, use the one from the archive.

      String tplName = cfgParser.getParameter("TEMPLATE", "Template");

      Reader r = null;

      if (tplName == null)
      {
        String s = getClass().getName();

        s = s.substring(0, s.lastIndexOf('.')).replace('.', '/') + "/template/fdl2" + ext() + ".tpl";

        r = new InputStreamReader(getClass().getClassLoader().getResourceAsStream(s));
      }
      else r = new FileReader(tplName);

      BufferedReader skel = new BufferedReader(r);

      // Copy all lines from the template to the output stream except for the tag line. For this line invoke the
      // generateXXX() method of the visitor.

      while (skel.ready())
      {
        String s = skel.readLine();

        if (tag().equals(s)) // Just read the tag line, let the ParserVisitor do it's job.
        {
          // Note that appropriate method is resolved using introspection. So to add a new generation target,
          // e.g. an XSL stylesheet, all that has to be done is defining an XSL subclass of Make and to add
          // a generateXSL method to the used ParserVisitor.

          try
          {
            pv.getClass().getMethod("generate" + ext().toUpperCase(), new Class[] { })
                         .invoke(pv, new Object[] { });
          }
          catch (java.lang.reflect.InvocationTargetException xcpt) { throw (Exception)xcpt.getTargetException(); }
        }
        else if (s != null)  out.println(s);
      }

      success = true; // If reached, output was succesfully written

      skel.close();
      out.close();

      postGenerate(tgt);

      System.out.println("Done.");
    }
    finally
    {
      if (out != null) { out.close(); out = null; }

      if (!success && output != null) output.delete(); // success == false indicates that an exception occurred
    }
  }

  protected void postGenerate(String tgt) throws Exception { }

  /**
   * PrintWriter Factory
   */

  protected final PrintWriter getPrintWriter(File file) throws IOException
  {
    Writer fw = new FileWriter(file);

    // Check if a postprocessing filter is configured. If so, splice it into the output stream

    String filter = cfgParser.getParameter("FILTER", "Class");

    if (filter != null)
    {
      try
      {
        try
        {
          fw = (Writer)Class.forName(filter).getConstructor(new Class[] { Class.forName("java.io.Writer"),
                                                                          Class.forName("java.util.Hashtable") })
                                             .newInstance(new Object[] { fw, cfgParser.getParameters("SUBST") });
        }
        catch (java.lang.reflect.InvocationTargetException xcpt) { throw xcpt.getTargetException(); }
      }
      catch (Throwable error)
      {
        System.out.println(eol + error.getMessage() + eol + name() + ": Unexpected error." + eol);

        error.printStackTrace();
      }
    }

    return new PrintWriter(fw);
  }

  /**
   * Returns the default output PrintWriter
   */

  protected final PrintWriter getPrintWriter() { return out; }

  /**
   * Returns the configuration data
   */

  protected final Hashtable getConfig() { return cfgParser.getTopics(); }

  /**
   * Class to be used as Visitor
   */

  protected abstract Class visitor();

  /**
   * File extension for generated output, used to locate configuration, too
   */

  protected abstract String ext();

  /**
   * Tag string in skeleton file
   */

  protected abstract String tag();

  /**
   * Name of generator, used in reports
   */

  protected abstract String name();

  /**
   * Explanation for command line parameters
   */

  protected abstract String usage();

  protected static final String eol = System.getProperty("line.separator", "\n");

  private FdlCompilationUnit cu;
  private String             src;
  private String             tgt;
  private PrintWriter        out = null;

  private static com.ibm.workflow.util.cfg.Parser cfgParser = null;
}
